{# Please consider keeping a link to the Firmant homepage in your footer -#}
                <p>Copyright &copy; 2010 Robert Escriva &brvbar;
                Powered by <a href="http://firmant.org/">Firmant</a></p>
