{% extends 'posts/archive_base.html' %}

{% block prevpage %}Previous Page (Yearly View){% endblock %}
{% block nextpage %}Next Page (Yearly View){% endblock %}
{% block prevcal %}Newer Posts (Grouped by Year){% endblock %}
{% block nextcal %}Older Posts (Grouped by Year){% endblock %}
