{% set doctitle = 'Document Title (in &lt;head&gt;)' %}
{% set divdocid = 'doc3' %}{# YUI doc id for the page div #}
{% set divdocclass = 'yui-t5' %}{# YUI doc class for the page div #}
{% set sitetitle = 'Site Title (in the body)' %}
{% set tagline = 'Something that describes your site in a few words' %}
{% set stylesheet = urlfor(None, type='static', path='css/style.css') %}
{% set primaryatomfeed = urlfor('atom', type='feed', slug='all') %}
