{% extends 'posts/archive_base.html' %}
{% import 'config.html' as config with context %}

{% block headers %}
{{ super() }}
        <link href="{{ config.primaryatomfeed }}" rel="alternate" title="Atom Feed" type="application/atom+xml" />
{% endblock %}
