{% import 'config.html' as config with context %}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
    <head>
        {% block headers %}
        <meta http-equiv="content-type" content="text/html; charset=utf-8" />

        <link rel="stylesheet" type="text/css" href="http://yui.yahooapis.com/2.8.0r4/build/reset-fonts-grids/reset-fonts-grids.css">
        <link rel="stylesheet" type="text/css" href="http://yui.yahooapis.com/2.8.0r4/build/base/base-min.css">

        <link rel="stylesheet" href="{{ urlfor(None, type='static', path='css/style.css') }}" type="text/css" />
        {% endblock %}

        <title>{% block documenttitle %}{{ config.doctitle }}{% endblock %}</title>
    </head>
    <body>
        <div id="{{ config.divdocid }}" class="{{ config.divdocclass }}">
            <div id="hd">
                <h1 class="title">
                    <a href="{{ urlfor(None) }}" title="Home">
                        {{ config.sitetitle }}
                    </a>
                </h1>
                <div class="tagline">{{ config.tagline }}</div>
            </div><!-- hd -->
            <div id="bd">
                <div id="yui-main">
                    <div class="yui-b">
                        {%- block content %}{% endblock %}
                    </div><!-- yui-b -->
                </div><!-- yui-main -->
                {% include 'sidebar.html' %}
            </div><!-- bd -->
            <div id="ft">
                {% include 'footer.html' %}
            </div><!-- ft -->
        </div><!-- doc3 -->
    </body>
</html>
