                <div class="yui-b sidebar">
                {%- block sidebar %}
                {% include 'sideboxes/_recent_posts.html' %}
                {% include 'sideboxes/_daily_archives.html' %}
                {% include 'sideboxes/_monthly_archives.html' %}
                {% include 'sideboxes/_yearly_archives.html' %}
                {% endblock %}
                </div><!-- yui-b sidebar-->
